@@ -1,17 +1,10 @@
 from typing import List, Dict, Any
 
-from sortedcontainers import SortedList
-
 from fakeredis import _msgs as msgs
 from fakeredis._command_args_parsing import extract_args
 from fakeredis._commands import command, CommandItem, Int, Key, Float
 from fakeredis._helpers import SimpleString, SimpleError, OK, Database
-
-
-class TDigest(SortedList):
-    def __init__(self, compression: int = 100):
-        super().__init__()
-        self.compression = compression
+from fakeredis.model import TDigest
 
 
 class TDigestCommandsMixin:
